/* Copyright © UCLES, 2013 */

/*global ulib, window*/
var mapping = (function ($) {
    var mapping = {};
    var prefix = "/";
    qti.mappingState = "ConnectPlus";

     
    // Send XML results to server
    function onError (s, st) {
        //console.log(s);
        throw new Error(st); 
    }
    
    (function () {
        var responseList = $.createHash();
        var allResults = $.createHash();
    
        function ISODateString(d) {
            function pad (n) { 
                return n < 10 ? '0' + n : n;
            }
    
            return d.getUTCFullYear() + '-' + 
                pad(d.getUTCMonth() + 1) + '-' +
                pad(d.getUTCDate()) + 'T' +
                pad(d.getUTCHours()) + ':' + 
                pad(d.getUTCMinutes()) + ':' + 
                pad(d.getUTCSeconds()) + 'Z';
        }
    
        function sendResults (id, responseVariable, onSuccess) {
    		var url = prefix + "response/" + id;
            var baseType = responseVariable.getBaseType();
            var cardinality = responseVariable.getCardinality();
            var state = responseVariable.getState();
            var rawValueObj = responseVariable.getValue();
    
            var data = qti.createSerializedXMLResults(id, baseType, cardinality, state, rawValueObj);
            $.postXML(url, onSuccess, data, onError); 
        }
    
        function sendCISResults (id, fldname, responseVariable, onSuccess) {
    		var seqElm = $.el("", "sequence-number"),
    		rspIdElm = $.el("", "response-identifier"),
    		url = prefix + "candidateInformationResponse/" + fldname,
            baseType = responseVariable.getBaseType(),
            cardinality = responseVariable.getCardinality(),
            state = responseVariable.getState(),
            rawValueObj = responseVariable.getValue(),
            data = qti.createSerializedXMLResults(id, baseType, cardinality, state, rawValueObj);
    
    		rspIdElm.text(fldname);
    		data.prepend(rspIdElm);
    		seqElm.text(responseVariable.sequenceNumber);
    		data.prepend(seqElm);
    
            $.postXML(url, onSuccess, data, onError);
        }
    
        var sendAllResults = function (callback) {
            $.forEach(allResults, function (response) {
                var id = response.key();
                var val = response.value();              
                var onSuccess = (function () {
                    var rid = id;
                    var arr = allResults;
                    return function () {
                        arr.remove(rid);
                    };
                })();
    
    			var ids = id.split(/\./);
    			if (ids[0] === 'candidateInformation') {
    				sendCISResults(id, ids[1], val, onSuccess);
    			} else {
    				sendResults(id, val, onSuccess);
    			}
            });            
    
            var reTest = function () {
                if (allResults.size() !== 0) {
                    sendAllResults(callback);
                } else {
                    if (typeof(callback) === "function") {
                        callback();
                    }
                }
            };
            setTimeout(reTest, 5000);
        };
    
        var prepAllResults = mapping.prepAllResults = function (callback) {
            var allResultsList = qti.getVariables();
            $.forEach(allResultsList, function (result) {
                allResults.set(result.getIdentifier(), result);
            });
            sendAllResults(callback);
        };
    
        function checkForResults() {
            if (responseList.size() === 0) {
                return false;
            }
    
            $.forEach(responseList, function (response) {
                var id = response.key();
                var val = response.value();        
                var onSuccess = (function () {
                    var rid = id;
                    return function () {
                        responseList.remove(rid);
                    };
                })();
    			var ids = id.split(/\./);
    			if (ids[0] === 'candidateInformation') {
    				sendCISResults(id, ids[1], val, onSuccess);
    			} else {
    				sendResults(id, val, onSuccess);
    			}
            });
    
            return true;
        }
    
        function changeResponseHandler(response) {        
            responseList.set(response.getIdentifier(), response);
        }
    
        //prepAllResults();
    
        qti.addToPollingList(checkForResults);
        qti.subscribeToEvent("changeResponseVariable", changeResponseHandler, "Adds new responses to a queue (when the response variable has been changed) to be sent to RM every 5 seconds - serverMapping", "serverMapResponses"); 
        qti.subscribeToEvent("changeState", changeResponseHandler, "Adds new responses to a queue (when the state has changed) to be sent to RM every 5 seconds - serverMapping", "serverMapResponsesState"); 
    }());
     
    // get XML results from server
    (function () {
        // Recover full results set
        function setResults(resultObj) {
            var variable = qti.getVariable(resultObj.id);
            var vBaseType = variable.getBaseType();
            var vCardinality = variable.getCardinality();
    
            if (vBaseType !== resultObj.baseType) {
                throw new Error("Base types do not match");
            }
            if (vCardinality !== resultObj.cardinality) {
                throw new Error("Cardinality does not match");
            }
            variable.setState(resultObj.state);
            variable.setValue(resultObj.value);
        }
    
        function handleItemVariables(itemResult) {
            var itemVariables = $(itemResult).find("|response");
            itemVariables.each(function(index, itemVariable) {
                var resultObj = qti.deserializeXML($(itemVariable));
                setResults(resultObj);
            });
        }
    
        function handleItemResults(xml) {
            var itemResults = $(xml).find("|Response");
            itemResults.each(function(index, itemResult) {
                var responseData = $(itemResult).find("|ResponseData"),
    			cdata = responseData.text(),
    			qtiResponse = (new DOMParser()).parseFromString(cdata, "text/xml");
    			handleItemVariables(qtiResponse);
            });
        }
    
        function handleCISResults(xml) {
    		var responsesXml = $(xml).find("candidate-information-results > response");
    		responsesXml.each(function(index, responseXml) {
    			var rspVal = $(responseXml).find("response-value").text(),
    			rspId = $(responseXml).find("response-identifier").text(),
    			variable = qti.getVariable("candidateInformation."+rspId),
    			vCardinality = variable.getCardinality();
    			if (vCardinality === "multiple") {
    				rspVal = rspVal.split(",");
    			}
    			variable.setValue(rspVal);
    		});
        }
    
        mapping.recoverAllXMLResults = function(url, callback) {
            $.getXML(url, handleItemResults, onError).done(callback);
        };
    
        mapping.recoverAllCISXMLResults = function(url, callback) {
            $.getXML(url, handleCISResults, onError).done(callback);
        };
    }());
     
    //  get JSON Results
    (function () {
        var errorCount = 0;
    
        function addToErrorCount () {
            errorCount += 1;
        }
    
        function zeroErrorCount () {
            errorCount = 0;
        }
        
        function testErrorState () {
            if (errorCount >= 3) {
                mapping.setStatus(5, "internal");
                zeroErrorCount();
            }
        }
    
        function handleExternalState (jsonResult) {
            var externalState = jsonResult.Status;
            mapping.setStatus(externalState, "external");
        }
    
        function connectError(s, st) {
            addToErrorCount();
            testErrorState();
        }
        
        function getTestStatus() {
            $.ulibGetJSON(prefix + "teststatus?type=JSON", handleExternalState, connectError);
        }
    
        qti.addToPollingList(getTestStatus);        
    }());
    
    //  TEST STATE CLASS
    (function () {
        var state = 0;
        var internalState = 0;
        var externalState = 0;
        var statusObjects = [];
        var timer;
        var preTestUrl = "instructions/preTest.xml.xhtml";
        var holdingPagesUrl = "instructions/holdingPages.xml.xhtml";
        var instructionsUrl = "instructions/instructions.xml.xhtml";
        var cisFormUrl = "instructions/cisForm.xml.xhtml";
        var holdingPagesContent;
        var _domHolder_main;
        var _domHolder_pausePage;
        var previousState = null;
        var candidateTime;
    
        var statusKey = {
            "NotStarted": 0,
            "CISFormShowing": 0.5,
            "CandidateInstructionsShowing": 1,
            "ReadingTimeShowing": 2,
            "NotifyInstructionsRead": 3,
            "Running": 4,
            "Paused": 5,
            "Ended": 6,
            "TestHostConfirmedTestEnded": 7,
            "AuditLogUploadCompletedByCandidate": 8,
            "AdminRequestedLogOff": 9, // when paused, Connect plus sends this to signal a log off
            "LoggedOff": 10, // confirmation the candidate is now logged off
            "getKeyFromValue": function (num) {
                if (isNaN(num)) {
                    throw new Error("Incorrect argument type for a statusKey");
                }
    
                for (var key in statusKey) {
                    if (statusKey.hasOwnProperty(key) && typeof statusKey[key] !== "function") {
                        if (statusKey[key] === num) {
                            return key;
                        }
                    }
                }
                return "ErrorState";
            }
        };
    
        // private functions to be called as state changes
        var main = function () {
            if (!_domHolder_main) {
                _domHolder_main = $('[role~="main"]');
            }
            return _domHolder_main;
        };
    
        var pausePage = function () {
            if (!_domHolder_pausePage) {
                _domHolder_pausePage = holdingPagesContent.find('[data-type="paused"]');
            }
            return _domHolder_pausePage;
        };
    
        function isInt(value) {
            return (value.toString().search(/^-?[0-9]+$/) === 0);
        }
    
        function timerCheck (t) {
    		var jsonVal = '{"RemainingTime": ' + t + '}';
    
            $.postText(prefix + "teststate/time", function(){}, t, onError);
    
            if (isInt(t)) {
                $.postJSON(prefix + "time?type=JSON", function(){}, jsonVal, onError);
            }
    
            if (t <= 0) {
                mapping.setStatus(7, "internal");
                qti.timerPause();
            }
        }
    
        qti.subscribeToEvent("time", timerCheck, "Sends the time to connect every minute", "sendTime");
    
        function ResendStatus () {
            var shell = this;
            this.timer = function (testStatus) {
                shell.resend = setTimeout(function () { sendStatus(testStatus); }, 1800);
            };
    
            this.stopTimer = function () {
                clearInterval(this.resend);
            };
        }
    
        var sendStatus = mapping.sendStatus = function (testStatus) {
            var jsonVal = '{"Status": "' + statusKey.getKeyFromValue(testStatus) + '"}';
            var resender = new ResendStatus();
            resender.timer(testStatus);
            var specificError = function (st) {
                resender.stopTimer();
                throw new Error("Error sending test state");
            };
    
            $.postJSON(prefix + "testStatus?type=JSON", function(){ resender.stopTimer(); }, jsonVal, specificError);
        };
    
    
        // STATE 0 FUNCTIONS
        var candidate;
    	function setCandidateDetails (count) {
    		var cdtId, wait, name, dob,
    		c = (count) ? count + 1 : 1;
    		if (c > 10) {
    			throw new Error("Error processing candidate details");
    		}
    		cdtId = $('[candidate-data="id"]');
    		if (!cdtId.length) {
    			wait = setTimeout( function () { setCandidateDetails(c); }, 200);
    		}
    		name = candidate.FirstName + " " + candidate.LastName;
    		$('[aria-labelled-by="candidate-name-label"]').text(name);
    		$('[aria-labelled-by="candidate-number-label"]').text(candidate.CandidateId);
    		$('[candidate-data="name"]').text(name);
    		
    		dob = new Date(candidate.DateOfBirth);
    		if ( isNaN(dob.valueOf()) ){
    			$('[candidate-data="dob"]').text('Invalid date "' + candidate.DateOfBirth + '"');
    		}
    		else {
    			$('[candidate-data="dob"]').text($.datepicker.formatDate('dd/mm/yy', dob));
    		}
    		
    		cdtId.text(candidate.CandidateId);
    	}
    
        qti.subscribeToEvent("candidate", setCandidateDetails, "Fill all candidate details with the correct values", "candidateDetails");
    
        function setCandidate (candidateJSON) {
            candidate = candidateJSON;
            qti.fireEvent("candidate", candidate);
        }
    
        function getHoldingPagesContent (xml) {
            holdingPagesContent = xml;
        }
    
        function showPreTestScreens (xml) {
            qti.fireEvent("preTestXML", xml);
            var screens = xml.find("[data-section]");
            if (screens.length < 1) {
                mapping.setStatus(1, "internal");
            }
            var contentArr = [];
            var screenNumber = 0;
            var contentHolder = main();
    
            function moveToNextScreen () {
    			var screen, screenType,
    			audioContent = contentHolder.find('#sample-audio-content');
    			if (audioContent.length > 0 && qti.audioPlayer) {
    				qti.audioPlayer.jPlayer("stop");
    			}
    			if (contentArr[screenNumber]) {
    				screen = contentArr[screenNumber];
    				screenType = screen.attr("data-sectionType");
    
    				if (screenType === "access" && !qti.hasAccessAPI) {
    					screenNumber += 1;
    					moveToNextScreen(); // skip the access screen
    					return;
    				}
    
    				contentHolder.empty();
    				contentHolder.append(screen);
    
    				screenNumber += 1;
    				//setCandidateConfirmDetails();
    
    				if (screenType === "access") {
    					qti.fireEvent("accessControls");
    				}
    				qti.fireEvent("preTestScreenChange");
    			} else if (screenNumber === contentArr.length) {
    				screen = holdingPagesContent.find('[data-type="wait"]').clone();
    				contentHolder.empty();
    				contentHolder.append(screen);
    				//mapping.setStatus(1, "internal");
    				if (qti.showCISForm) {
    					mapping.setStatus(0.5, "internal");
    				} else {
    					mapping.setStatus(1, "internal");
    				}
    			}
            }
    
            screens.each(function(index, screen) {
                screen = $(screen);
                screen.find("[data-confirm]").click(moveToNextScreen);
                contentArr.push(screen);
            });
    
            moveToNextScreen();
        }
    
        // STATE ! FUNCTIONS
        function showInstructions (xml) {
            qti.fireEvent("instructionsStart", xml);
            var content = main();
            var items = xml.find("[data-section]");
            if (items.length < 1) {
                content.empty();
    			mapping.setStatus(3, "internal");
            }
            var screens = [];
            //var screenNumber = 0;
    
            items.each(function(index, item) {
                item = $(item);
                screens.push(item);
                //screenNumber += 1;
            });
    
            /* if (screens.length !== 2) {
                throw new Error("Incorrect item number in Instructions");
            } */
    
            function confirmReading () {
    			var screen = holdingPagesContent.find('[data-type="wait"]').clone();
    			content.empty();
    			content.append(screen);
    			mapping.setStatus(3, "internal");
            }
            var button = screens[0].find("#instructionsConfirm");
            button.click(confirmReading);
    
            function showCopyrightInfo () {
                var copy = $(document).find('[connect|class~="copyrightInfo"]');
                copy.css({"display": "block"});
            }
            var copyright = xml.find('[connect|class~="showCopyright"]');
            copyright.click(showCopyrightInfo);
    
            content.empty();
            content.append(screens[0]);
            qti.fireEvent("showInstructions");
        }
    
    	// CIS STATE FUNCTIONS
    	function showCISForm (xhtml) {
    		var content = main(),
    		cisform = xhtml.find('form'),
    		confirmAction = function() {
    			var screen = holdingPagesContent.find('[data-type="wait"]').clone();
    			content.empty();
    			content.append(screen);
    			mapping.setStatus(1, "internal");
    		},
    		button = cisform.find('#finish_button');
    		button.click(confirmAction);
    
    		content.empty();
    		content.append(cisform);
    		qti.setupCISForm();
            qti.fireEvent("showCISForm");
    	}
    
        // STATE 4 FUNCTIONS
        //function setLocalTimer () {
        function setCandidateTime () {
            var testTime = $('connect|timeLimits').attr('maxTime');
            var addTime = Number(candidate.AdditionalTime);
            var additionalTime = (addTime) ? addTime * 60 : 0;
            candidateTime = Number(testTime) + additionalTime;
    
            //qti.timerStart(candidateTime);
        }
    
    	function resetTestTimer(t) {
            t = Number(t);
            timer = t * 60;
    
            if (timer === "NaN") {
                timer = candidateTime;
            }
            qti.timerStart(timer);
    
    		if (timer && qti.audioPlayer && $('#main-audio-content').length > 0) {
    			qti.audioPlayer.mainAudioStart = candidateTime - timer;
    		}
        }
    
        function testStart () {
            qti.fireEvent("testStart");
        }
    
        function endTest() {
            mapping.setStatus(7, "internal");
            qti.timerPause();
            return false;
        }
    
        function setEndTest () {
            $("#endTest").bind("click", endTest);
        }
    
        function responsesDataReady () {
            qti.fireEvent("responsesDataReady");
        }
    
        function checkRecovery () {
            if (candidate.TestRecoveryStatus === "NoRecovery") {
                mapping.prepAllResults(responsesDataReady);
                return; //This is a fresh test - do nothing
            } else if (candidate.TestRecoveryStatus === "RecoverySuccess") {
                //console.log("Handle stored data recovery");
                mapping.recoverAllXMLResults(prefix + "response/");
    			if (qti.showCISForm) {
    				mapping.recoverAllCISXMLResults(prefix + "candidateInformationResponse/", responsesDataReady);
    			}
                return;
            } else if (!candidate.TestRecoverySuceeded) {
                //throw new Error("Incomplete Recovery Data");
                console.log("Incomplete Recovery Data");
    			return;
            }
            throw new Error("Unknown test recovery type");
        }
    
    	if (qti.showCISForm) {		
    		qti.subscribeToEvent("showCISForm", checkRecovery, "check and recover saved responses or upload all responses", "checkRecovery");
    	} else {		
    		qti.subscribeToEvent("instructionsStart", checkRecovery, "check and recover saved responses or upload all responses", "checkRecovery");
    	}
    
        // STATE 5 FUNCTIONS
        function pauseTimer () {
            qti.timerPause();
    		if (qti.audioPlayer && $('#main-audio-content').length > 0) {
    			qti.audioPlayer.jPlayer("pause");
    		}
        }
    
        function showPauseHoldingPage() {
    		var content = main(),
    		pauseScreen = pausePage();
            if (qti.preloadTestContents) {
    			if (content.find('[data-type="paused"]').length === 0) {
    				content.find('>:not([connect|class~="itemBody"])').remove();
    				content.find('[connect|class="itemBody"]').not('[connect|identifier]').remove();
    				content.append(pauseScreen);
    			}
    			content.find('>[connect|class~="activeItem"]').removeAttrToken('connect:class', 'activeItem').hide();
    			pauseScreen.addAttrToken('connect:class', 'activeItem').show();
    		} else {
    			content.empty();
    			content.append(pauseScreen);
    		}
        }
    
        function showTestControls() {
            $('[role~="navigation"]').show();
            $('[role~="toolbar"]').show();
            $('#timer').show();
            if (qti.hasAccessAPI) {
                $('[connect|class~="accessibilityLink"]').show();
                $('[connect|class~="settingsLink"]').show();
            }
        }
    
    	function hideTools(){
    		$('div#context-menu-layer').hide();
    		$('ul.context-menu-list').hide();
    		$('div.note').hide();
    		$('div.dialogue-overlay').hide();
    		$('div.dialogue-wrapper').hide();
    		$('[connect|class~="floatingToolContainer"]').css("visibility", "hidden"); //Need to bring inline with other "hide mechanisms
    	}
    
    	function hideTestControls() {
    		$('[role~="navigation"]').hide();
    		$('[role~="toolbar"]').hide();
    		$('#timer').hide();
    		$('[connect|class~="accessibilityLink"]').hide();
    		$('[connect|class~="settingsLink"]').hide();
    	}
    
        // STATE 7 FUNCTIONS
        function showFinishedHoldingPage() {
            var main = $("[role~='main']");
            main.empty();
            var finishPage = holdingPagesContent.find('[data-type="finished"]');
            main.append($(finishPage));
        }
    
        // STATE 0
        statusObjects[0] = {
            rules: function (source) {
                if (internalState === 5) {
                    this.changeState(5);
                }
                if (qti.showCISForm) {
                    if (internalState === 0.5) {
                        this.changeState(0.5);
                    }
                } else if (instructionsUrl) {
                    if ((statusKey[externalState] === 1 || statusKey[externalState] === 4) && (internalState === 1)) {
                        this.changeState(1);
                    }
                } else {
                    if (statusKey[externalState] === 4 && (internalState === 1)) {
                        this.changeState(4);
                    }
                }
            },
            changeState: function (newState) {
                this.hasRun = true;
                internalState = newState;
                previousState = state;
                state = newState;
                statusObjects[newState].initState();
            },
            initState: function () {
                $.getXML(preTestUrl, showPreTestScreens, onError);
                $.ulibGetJSON(prefix + "candidate?type=JSON", setCandidate, onError);
                $.getXML(holdingPagesUrl, getHoldingPagesContent, onError);
                hideTestControls();
            },
            hasRun: false
        };
    
        // STATE 0.5
        statusObjects[0.5] = {
            rules: function (source) {
                /* if (internalState === 1) {
                    this.changeState(internalState);
                } */
                if (instructionsUrl) {
                    if ((statusKey[externalState] === 1 || statusKey[externalState] === 4) && (internalState === 1)) {
                        this.changeState(1);
                    }
                } else {
                    if (statusKey[externalState] === 4 && (internalState === 1)) {
                        this.changeState(4);
                    }
                }
            },
            changeState: function (newState) {
                this.hasRun = true;
                internalState = newState;
                previousState = state;
                state = newState;
                statusObjects[newState].initState();
            },
            initState: function () {
                try {
                    $.getXML(cisFormUrl, showCISForm, onError);
                } catch (er) {
                    mapping.setStatus(1, "internal");
                }
            },
            hasRun: false
        };
    
        // STATE 1
        statusObjects[1] = {
            rules: function (source) {
                if (internalState === 3 || internalState === 5) {
                    this.changeState(internalState);
                }
                if (statusKey[externalState] === 6) {
    				this.changeState(7);
                }
            },
            changeState: function (newState) {
                this.hasRun = true;
                previousState = state;
                state = newState;
                statusObjects[newState].initState();
            },
            initState: function () {
                $.getXML(instructionsUrl, showInstructions, onError);
            },
            hasRun: false
        };
    
        // STATE 3
        statusObjects[3] = {
            rules: function (source) {
                if (internalState === 3) {
                    this.unLocked = true;
                }
                if (statusKey[externalState] === 4) {
                    this.changeState(4);
                }
                if (internalState === 5) {
                    this.changeState(5);
                }
                if (statusKey[externalState] === 6) {
                    this.changeState(7);
                }
            },
            changeState: function (newState) {
                this.hasRun = true;
                internalState = newState;
                previousState = state;
                state = newState;
                sendStatus(3);
                statusObjects[newState].initState();
            },
            initState: function () {
                // no functions for state 3
            },
            hasRun: false
        };
    
        // STATE 4
        statusObjects[4] = {
            rules: function (source) {
                if (statusKey[externalState] === 5 || internalState === 5) {
                    this.changeState(5);
                }
    
                if (internalState === 7 || statusKey[externalState] === 6) {
                    this.changeState(7);
                }
    
                if (statusKey[externalState] === 9) {
                    this.changeState(9);
                }
            },
            changeState: function (newState) {
                this.hasRun = true;
                internalState = newState;
                previousState = state;
                state = newState;
                statusObjects[newState].initState();
            },
            initState: function () {
                var onError = function () {
    				qti.timerStart(candidateTime);
                }, tPromise;
    
                if (this.hasRun) {
                    // Resume test
    				if (qti.preloadTestContents) {
    					qti.displayItem();
    				} else {
    					qti.changeItem(qti.getCurrentItem());
    				}
                    //$.getText(prefix + "testState/time", resetTestTimer, onError);
                    $.get(prefix + "testState/time", resetTestTimer).done(function() {
    					if (qti.audioPlayer && $('#main-audio-content').length > 0) {
    						qti.audioPlayer.jPlayer("play");
    					}
    				}).error(onError);
                } else {
    				setCandidateTime();
                    // test if there has been a machine change or test restart
                    if (candidate.TestRecoveryStatus === "RecoverySuccess") {
                        //$.get(prefix + "testState/time", resetTestTimer).error(onError);
                        tPromise = $.getText(prefix + "testState/time", resetTestTimer, onError);
                    } else {
                        //setLocalTimer();
    					qti.timerStart(candidateTime);
                    }
    				$.when(tPromise).always(function() {
    					// Initialise test
    					if (qti.preloadTestContents) {
    						qti.loadTestItems(testStart);
    					} else {
    						qti.changeItem(qti.getFirstItem(), testStart);
    					}
    				});
                    //setEndTest();
                }
                showTestControls();
            },
            hasRun: false
        };
    
        // STATE 5
        statusObjects[5] = {
            rules: function (source) {
                if (statusKey[externalState] === 5) {
                    return;
                }
    
                if (statusKey[externalState] === 6) {
                   this.changeState(7);
                }
    
                if (statusKey[externalState] === 4 || statusKey[externalState] === 0) {
                    this.changeState(previousState);
                }
    
                if (statusKey[externalState] === 9) {
                    this.changeState(9);
                }
            },
            changeState: function (newState) {
                this.hasRun = true;
                internalState = newState;
                state = newState;
                statusObjects[newState].initState();
            },
            initState: function () {
                pauseTimer();
                showPauseHoldingPage();
    			hideTools();
    			hideTestControls();
            },
            hasRun: false
        };
    
        // STATE 7
        statusObjects[7] = {
            rules: function (source) {
                if (statusKey[externalState] === 8) {
                    this.changeState(8);
                }
            },
            changeState: function (newState) {
                this.hasRun = true;
                internalState = newState;
                state = newState;
                statusObjects[newState].initState();
            },
            initState: function () {
                showFinishedHoldingPage();
    			hideTools();
                hideTestControls();
    			qti.fireEvent("testEnd");
                mapping.prepAllResults(function () { mapping.sendStatus(7); });
            },
            hasRun: false
        };
    
    	// STATE 8
    	statusObjects[8] = {
    		rules: function (source) {
    			//return;
    			if (statusKey[externalState] === 8) {
    				this.initState();
    			}
    			if (statusKey[externalState] === 9) {
    				this.changeState(9);
    			}
    		},
    		changeState: function (newState) {
    			//return;
    			this.hasRun = true;
    			internalState = newState;
    			state = newState;
    			statusObjects[newState].initState();
    		},
    		initState: function () {
    			//window.close();
    			window.location.href = "/";
                
    			//console.log("Run any shutdown scripts - thank you and good night");
    			// RUN SHUTDOWN FUNCS
    		},
    		hasRun: false
    	};
    
        // STATE 9
        statusObjects[9] = {
            rules: function (source) {
                if (statusKey[externalState] === 9) {
                    this.initState();
                }
                if  (statusKey[externalState] === 4) {
                    this.changeState(4);
                }
            },
            changeState: function (newState) {
                this.hasRun = true;
                internalState = newState;
                state = newState;
                statusObjects[newState].initState();
            },
            initState: function () {
                var onError = function () {
                    // This will automatically retry every 5
                    // seconds if the state has not been changed.
                };
                var onSuccess = function () {
                    //window.close();
    				window.location.href = "/";
                };
    
                $.postText(prefix + "candidatelogout", onSuccess, "null", onError);
            },
            hasRun: false
        };
    
    
        /*
        statusObjects[1999] = {
            rules: function () {
                // Rules for changing state
                // if state === receivedState return
                // is the received state legal for this state?
                //  do the state and received state allow progression?
                //  if so, call changeState
            },
            changeState: function () {
                // functions to be called on state change - including set new state value
                // set hasRun
                // run any specific functions - none forseen at present
                // run initState for new state object
                //  set state
            },
            initState: function () {
                // functions to be called on initiation of this state
                // these may depend on the value of hasRun
            },
            hasRun: false
        };
        */
    
        mapping.setStatus = function (newState, source) {
            //console.log("state:" + state);
            //console.log("internalState: " + internalState);
            //console.log("externalState: " + externalState);
    
            if (source === "internal") {
                internalState = newState;
            } else if (source === "external") {
                externalState = newState;
            } else {
                throw new Error("Incorrect state received");
            }
            statusObjects[state].rules();
        };
    
        mapping.getStatus = function () {
            return state;
        };
    
        // Always run state 0 initialisation automatically
        qti.mappingLoadListener.push(statusObjects[0].initState);
    
    }());

    return mapping;
}(ulib.withNamespaces("http://www.w3.org/1999/xhtml", {
        connect: "http://connect.ucles.org.uk/ns/ConnectDeliveryEngine",
        qti: "http://www.imsglobal.org/xsd/imsqti_v2p1",
        p: "http://connect.ucles.org.uk/ns/QTIItemParameters",
        svg: "http://www.w3.org/2000/svg"})));

