/* Copyright © UCLES, 2009, 2010 */

/*global ulib, window, qti*/

var theme = (function ($) {
    var theme = {};

    
    (function () {
    
        var flashChanges = 20,
            firstAlarm = false,
            secondAlarm = false;
    
        function timerMarkup () {
            qti.timerMarkup.display = $('span[connect|function~=timer]');
            qti.timerMarkup.displayValue = $('span[connect|function~=timer] > em');
            qti.timerMarkup.displayText = $('span[connect|function~=timer] > span');
            
            //Show and Hide Timer Display
            var clockState = true;
            var clickMessage = $.ulib.span().text("Click to show time");
            clickMessage.attr("connect:function", "clickTimer");
            
            $('#timer').click(function(event){
                if (clockState){
                    qti.timerMarkup.display.css("display", "none");
                    qti.timerMarkup.display.parent().append(clickMessage);
                    clockState = false;
                } else{
                    clickMessage.remove();
                    qti.timerMarkup.display.css("display", "inline-block");
                    clockState = true;
                }
            });
        }
        qti.themeLoadListener.push(timerMarkup);
    	
    	if (!qti.hideTimerWarnings) {
    		var timeOutWarning = function(t) {
    			/* timeWarning will be triggered every 5 seconds - ensure the flashing lasts no longer */
    			var flashColor = '#f33', // light red - will be on a very dark background
    				originalColor = qti.computedStyle($('span[connect|function~=timer]')[0], 'color'),
    				color = originalColor,
    				flash = function(){
    					color = (color===originalColor) ? flashColor : originalColor;
    
    					if (flashChanges > 0){
    						qti.timerMarkup.display.css("color", color);
    						flashChanges-=1;
    						setTimeout(flash, 200);
    					}
    					else if (secondAlarm){
    						qti.timerMarkup.display.css("color", flashColor);
    					}
    				};
    			
    			if (t <= 10 && !firstAlarm){
    				setTimeout(flash, 0);
    				firstAlarm = true;
    			}
    			else if (t <= 5 && !secondAlarm){
    				secondAlarm = true;
    				setTimeout(flash, 0);
    			}
    		};
    		
    		qti.subscribeToEvent("time", timeOutWarning, "warn candidate at 10mins and 5mins", "timeOutWarning", true);
    	}
    }());
    
    (function () {
    
    	function prepareTestScreen () {
    		$('div#banner').hide();
    		qti._onResize();
    	}
    	qti.subscribeToEvent("testStart", prepareTestScreen, "Hide the top banner on starting the test", "prepareTestScreen");
    })();
    
    (function () {
        
        function toolbar () {
            var toolbox = $("#access-toolbar");
            var tools = toolbox.find("[class~=tool]");
            var toolWidth = 0;
            var dynamicStyles = $.createHash();
            var tabUp = true;
            
            //Determine direction of tabbing
            $(document).keypress(function(event){
    
                if (event.keyCode === 9){
                    if (!event.shiftKey){
                        tabUp = true;
                    } else {
                        tabUp = false;
                    }            
                }
            });
            
            tools.each(function(index, tool){
            
                tool = $(tool);
            
                var optionValue;
                var handle = tool.find("dt");
                var slideWidth = tool[0].offsetWidth;
                var toolType = handle.text();
                var toolOptions = tool.find("a");
                var toolFocused = false;
                var toolSelected = false;
    
                function setOption(){
                
                    if (optionValue.substring(0, 6) !== "volume"){
    
                        if (!dynamicStyles.has(toolType)) {
                            dynamicStyles.set(toolType, $.link());
                            $(dynamicStyles.get(toolType)).attr("rel", "stylesheet");
                            $("head").append(dynamicStyles.get(toolType));
                        }
                        $(dynamicStyles.get(toolType)).attr("href", optionValue + ".css");
                    } else{
                        var mediaVolume = 
                            Math.floor(Math.exp(parseInt(optionValue.substring(6), 10)/2)) * 5;
                        if (qti.mediaPlayer.exists){
                            qti.mediaPlayer.volume(mediaVolume);
                        }
                    }
                }
            
                function showTool(selectedOption){
                    
                    tool.parent().cssAnimation("width", toolWidth + "px", slideWidth + "px", 
                        250, {tweening: $.tweening.easeBoth()}).
                    then(function(){
                        toolOptions.attr("tabindex", "1");
                        if (selectedOption){
                            selectedOption.focus();
                        }
                        tool.addClass("open");
                        tool.addClass("extended");
                    }).start();
                }
                
                function hideTool(){
                    
                    tool.parent().cssAnimation("width", slideWidth + "px", toolWidth + "px", 
                    250, {tweening: $.tweening.easeBoth()}).
                    then(function(){
                        tool.removeClass("extended");
                        tool.removeClass("open");
                        toolSelected = false;
                        tool[0].blur();
                    }).start();
                }
                
                tool.css({left: "0", 
                    width: tool[0].offsetWidth + "px"});
                
                if (handle[0].offsetWidth > toolWidth){
                    toolWidth = handle[0].offsetWidth;
                }
                
                //The tool itself is tabbable. When this gets focus
                //set its tabindex to -1 and set the tabindex of all
                //options on the tool to 1. This mechanism prevents
                //the page scrolling and also means that you can step
                //between tools quickly without having to tab through
                //every option
                tool.focus(function (event) {
                
                    $.later(function(){
                        if (!toolSelected){
                        
                            toolSelected = true;    
                            tool.attr("tabindex", "-1");
                            
                            if (tabUp){
                                showTool(toolOptions[0]);
                            } else{
                                showTool(toolOptions[toolOptions.length - 1]);
                            }
                        }
                    });
                });
                
                toolOptions.each(function(index, toolOption){
                    toolOption = $(toolOption);
                    
                    toolOption.focus(function (event) {
                    
                        if (tool.hasClass("extended")){
                            toolFocused = true;
                        }
                    });
                    
                    //If focus is lost from the tool, set the tabindex
                    //on all options to -1 and set the tabindex on the
                    //tool to 1
                    toolOption.blur(function (event) {
                    
                        toolFocused = false;
                        $.later(function () {
                        
                            if (!toolFocused && tool.hasClass("extended")) {
    
                                toolOptions.attr("tabindex", "-1");
                                tool.attr("tabindex", "1");
                                hideTool();
                            }
                        });
                    });
                
                    toolOption.click(function(event){
                        toolOptions.removeClass("current");
                        toolOption.removeClass("toolOption");
                        optionValue = toolOption.attr("class");
                        toolOption.addClass("toolOption");
                        toolOption.addClass("current");
                        toolOption[0].blur();
                        qti.fireEvent("accessChange", toolOption);
                        setOption();
                        event.preventDefault();
                    });
                
                });
                tool.parent().css({width: toolWidth + "px"});
            });
        }
        qti.themeLoadListener.push(toolbar);    
    })();
    
    (function () {
        
    	$(document).ready(function() {
    		$('[role="main"]').attr('id', 'highlightable-content');
    		qti._setupTextHighlighting({noteHtext: false});
    	});
    
    	var itemLoadHandler = function() {
    		if (qti.getCurrentItem()) {
    			qti.updateScreenStore();
    			qti.restoreTextNotes();
    			qti.restoreSavedRanges();
    		}
    	};
    	var itemUnloadHandler = function() {
    		if (qti.getCurrentItem()) {
    			qti.saveTextNotes();
    			qti.saveCurrentRanges();
    		}
    	};
    
    	qti.subscribeToEvent("itemLoad", itemLoadHandler, "Restores any saved ranges, text notes and data relevant to the current item, from memory", "restoreTextData");
    	qti.subscribeToEvent("itemUnload", itemUnloadHandler, "Perform necessary saving and housekeeping for text ranges, notes and data for the currently displayed item", "saveTextData");
    
    	if (qti.useClipboard) {
    		qti.subscribeToEvent("newScreenDisplay", qti.setupTextareaOverlay, "Setup overlaid editable div in place of textareas", "setupTextareaOverlay");
    	}
    	if (qti.useExpandableTextEntryInteraction) {
    		qti.subscribeToEvent("newScreenDisplay", qti.setupExpandableTextEntryInteractionOverlay, "Setup overlaid expandable textEntryInteractions", "setupExpandableTextEntryInteractionOverlay");
    	}
    })();
    
    (function () {
    	var testPart,
    		endDialogue,
    		insertTestEndPause = function (){
    			var ul = $('div[connect|class="testPart"]>ul'),
    				li = $('<li/>', {'connect:class': 'assessmentSection', 'connect:identifier': 'SecEnd'}),
    				span = $('<span/>', {'class': "section-label"}).text('END'),
    				ul2 = $('<ul/>'),
    				li2 = $('<li/>', {'connect:class': 'assessmentItemRef', 'connect:identifier': 'test-end'}),
    				a = $('<a/>', {'class': 'test-end', href: 'test-content/test-end.xml.xhtml'}),
    				span2 = $('<span/>', {'class': 'question-label'}).text('End'),
    				span3 = $('<span/>', {'class': 'question-number', 'xml:space': 'default'}).text('End');
    				
    			span.appendTo(li);
    			a.append(span2);
    			a.append(span3);
    			a.appendTo(li2);
    			li2.appendTo(ul2);
    			ul2.appendTo(li);
    			li.appendTo(ul);
    			
    			a.click(function (event) {
    				endDialogue.showDialogue("End of test");
    				event.preventDefault();
    				return false;
    			});
    		},
    		buildEndDialogue = function () {
    			var endDialogue = {};
    			var openDialogue = function(dialogueName) {
    				$.dialogue({
    					title: dialogueName,
    					hasTopClose: true,
    					hideScreen:true,
    					buttons: {"Resume test": "closeDialogue"}, 
    					content: "<p>You have reached the end of the test. All your answers have been stored.</p>" +
    							"<p>You may still go back and change your answers. Click the button below to return to your test.</p>" +
    							"<p>If you wish to leave, please tell your invigilator.</p>",
    					img: "images/main/screen.png",
    					imgWidth: 48,
    					imgHeight: 48
    				});
    			};
    
    			endDialogue.showDialogue = function (dialogueName) {
    				openDialogue(dialogueName);
    			};
    
    			return endDialogue;
    		};
    	
    	if (qti.provideTestEndPause) {
    		endDialogue = buildEndDialogue();
    		qti.subscribeToEvent("testStart", insertTestEndPause, "insert a test end pause  button into the navigation", "insertTestEndPause", true);
    	}
    }());

    return theme;
}(ulib.withNamespaces("http://www.w3.org/1999/xhtml", {
        connect: "http://connect.ucles.org.uk/ns/ConnectDeliveryEngine",
        qti: "http://www.imsglobal.org/xsd/imsqti_v2p1",
        svg: "http://www.w3.org/2000/svg"})));

